import { Buffer } from 'buffer';
import { utils, getPublicKey, getSharedSecret, sign } from 'noble-secp256k1';
import base58 from 'micro-base58';
import { derToJose } from 'ecdsa-sig-formatter';

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var runtime_1 = createCommonjsModule(function (module) {
/**
 * Copyright (c) 2014-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

var runtime = (function (exports) {

  var Op = Object.prototype;
  var hasOwn = Op.hasOwnProperty;
  var undefined$1; // More compressible than void 0.
  var $Symbol = typeof Symbol === "function" ? Symbol : {};
  var iteratorSymbol = $Symbol.iterator || "@@iterator";
  var asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator";
  var toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";

  function define(obj, key, value) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
    return obj[key];
  }
  try {
    // IE 8 has a broken Object.defineProperty that only works on DOM objects.
    define({}, "");
  } catch (err) {
    define = function(obj, key, value) {
      return obj[key] = value;
    };
  }

  function wrap(innerFn, outerFn, self, tryLocsList) {
    // If outerFn provided and outerFn.prototype is a Generator, then outerFn.prototype instanceof Generator.
    var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator;
    var generator = Object.create(protoGenerator.prototype);
    var context = new Context(tryLocsList || []);

    // The ._invoke method unifies the implementations of the .next,
    // .throw, and .return methods.
    generator._invoke = makeInvokeMethod(innerFn, self, context);

    return generator;
  }
  exports.wrap = wrap;

  // Try/catch helper to minimize deoptimizations. Returns a completion
  // record like context.tryEntries[i].completion. This interface could
  // have been (and was previously) designed to take a closure to be
  // invoked without arguments, but in all the cases we care about we
  // already have an existing method we want to call, so there's no need
  // to create a new function object. We can even get away with assuming
  // the method takes exactly one argument, since that happens to be true
  // in every case, so we don't have to touch the arguments object. The
  // only additional allocation required is the completion record, which
  // has a stable shape and so hopefully should be cheap to allocate.
  function tryCatch(fn, obj, arg) {
    try {
      return { type: "normal", arg: fn.call(obj, arg) };
    } catch (err) {
      return { type: "throw", arg: err };
    }
  }

  var GenStateSuspendedStart = "suspendedStart";
  var GenStateSuspendedYield = "suspendedYield";
  var GenStateExecuting = "executing";
  var GenStateCompleted = "completed";

  // Returning this object from the innerFn has the same effect as
  // breaking out of the dispatch switch statement.
  var ContinueSentinel = {};

  // Dummy constructor functions that we use as the .constructor and
  // .constructor.prototype properties for functions that return Generator
  // objects. For full spec compliance, you may wish to configure your
  // minifier not to mangle the names of these two functions.
  function Generator() {}
  function GeneratorFunction() {}
  function GeneratorFunctionPrototype() {}

  // This is a polyfill for %IteratorPrototype% for environments that
  // don't natively support it.
  var IteratorPrototype = {};
  IteratorPrototype[iteratorSymbol] = function () {
    return this;
  };

  var getProto = Object.getPrototypeOf;
  var NativeIteratorPrototype = getProto && getProto(getProto(values([])));
  if (NativeIteratorPrototype &&
      NativeIteratorPrototype !== Op &&
      hasOwn.call(NativeIteratorPrototype, iteratorSymbol)) {
    // This environment has a native %IteratorPrototype%; use it instead
    // of the polyfill.
    IteratorPrototype = NativeIteratorPrototype;
  }

  var Gp = GeneratorFunctionPrototype.prototype =
    Generator.prototype = Object.create(IteratorPrototype);
  GeneratorFunction.prototype = Gp.constructor = GeneratorFunctionPrototype;
  GeneratorFunctionPrototype.constructor = GeneratorFunction;
  GeneratorFunction.displayName = define(
    GeneratorFunctionPrototype,
    toStringTagSymbol,
    "GeneratorFunction"
  );

  // Helper for defining the .next, .throw, and .return methods of the
  // Iterator interface in terms of a single ._invoke method.
  function defineIteratorMethods(prototype) {
    ["next", "throw", "return"].forEach(function(method) {
      define(prototype, method, function(arg) {
        return this._invoke(method, arg);
      });
    });
  }

  exports.isGeneratorFunction = function(genFun) {
    var ctor = typeof genFun === "function" && genFun.constructor;
    return ctor
      ? ctor === GeneratorFunction ||
        // For the native GeneratorFunction constructor, the best we can
        // do is to check its .name property.
        (ctor.displayName || ctor.name) === "GeneratorFunction"
      : false;
  };

  exports.mark = function(genFun) {
    if (Object.setPrototypeOf) {
      Object.setPrototypeOf(genFun, GeneratorFunctionPrototype);
    } else {
      genFun.__proto__ = GeneratorFunctionPrototype;
      define(genFun, toStringTagSymbol, "GeneratorFunction");
    }
    genFun.prototype = Object.create(Gp);
    return genFun;
  };

  // Within the body of any async function, `await x` is transformed to
  // `yield regeneratorRuntime.awrap(x)`, so that the runtime can test
  // `hasOwn.call(value, "__await")` to determine if the yielded value is
  // meant to be awaited.
  exports.awrap = function(arg) {
    return { __await: arg };
  };

  function AsyncIterator(generator, PromiseImpl) {
    function invoke(method, arg, resolve, reject) {
      var record = tryCatch(generator[method], generator, arg);
      if (record.type === "throw") {
        reject(record.arg);
      } else {
        var result = record.arg;
        var value = result.value;
        if (value &&
            typeof value === "object" &&
            hasOwn.call(value, "__await")) {
          return PromiseImpl.resolve(value.__await).then(function(value) {
            invoke("next", value, resolve, reject);
          }, function(err) {
            invoke("throw", err, resolve, reject);
          });
        }

        return PromiseImpl.resolve(value).then(function(unwrapped) {
          // When a yielded Promise is resolved, its final value becomes
          // the .value of the Promise<{value,done}> result for the
          // current iteration.
          result.value = unwrapped;
          resolve(result);
        }, function(error) {
          // If a rejected Promise was yielded, throw the rejection back
          // into the async generator function so it can be handled there.
          return invoke("throw", error, resolve, reject);
        });
      }
    }

    var previousPromise;

    function enqueue(method, arg) {
      function callInvokeWithMethodAndArg() {
        return new PromiseImpl(function(resolve, reject) {
          invoke(method, arg, resolve, reject);
        });
      }

      return previousPromise =
        // If enqueue has been called before, then we want to wait until
        // all previous Promises have been resolved before calling invoke,
        // so that results are always delivered in the correct order. If
        // enqueue has not been called before, then it is important to
        // call invoke immediately, without waiting on a callback to fire,
        // so that the async generator function has the opportunity to do
        // any necessary setup in a predictable way. This predictability
        // is why the Promise constructor synchronously invokes its
        // executor callback, and why async functions synchronously
        // execute code before the first await. Since we implement simple
        // async functions in terms of async generators, it is especially
        // important to get this right, even though it requires care.
        previousPromise ? previousPromise.then(
          callInvokeWithMethodAndArg,
          // Avoid propagating failures to Promises returned by later
          // invocations of the iterator.
          callInvokeWithMethodAndArg
        ) : callInvokeWithMethodAndArg();
    }

    // Define the unified helper method that is used to implement .next,
    // .throw, and .return (see defineIteratorMethods).
    this._invoke = enqueue;
  }

  defineIteratorMethods(AsyncIterator.prototype);
  AsyncIterator.prototype[asyncIteratorSymbol] = function () {
    return this;
  };
  exports.AsyncIterator = AsyncIterator;

  // Note that simple async functions are implemented on top of
  // AsyncIterator objects; they just return a Promise for the value of
  // the final result produced by the iterator.
  exports.async = function(innerFn, outerFn, self, tryLocsList, PromiseImpl) {
    if (PromiseImpl === void 0) PromiseImpl = Promise;

    var iter = new AsyncIterator(
      wrap(innerFn, outerFn, self, tryLocsList),
      PromiseImpl
    );

    return exports.isGeneratorFunction(outerFn)
      ? iter // If outerFn is a generator, return the full iterator.
      : iter.next().then(function(result) {
          return result.done ? result.value : iter.next();
        });
  };

  function makeInvokeMethod(innerFn, self, context) {
    var state = GenStateSuspendedStart;

    return function invoke(method, arg) {
      if (state === GenStateExecuting) {
        throw new Error("Generator is already running");
      }

      if (state === GenStateCompleted) {
        if (method === "throw") {
          throw arg;
        }

        // Be forgiving, per 25.3.3.3.3 of the spec:
        // https://people.mozilla.org/~jorendorff/es6-draft.html#sec-generatorresume
        return doneResult();
      }

      context.method = method;
      context.arg = arg;

      while (true) {
        var delegate = context.delegate;
        if (delegate) {
          var delegateResult = maybeInvokeDelegate(delegate, context);
          if (delegateResult) {
            if (delegateResult === ContinueSentinel) continue;
            return delegateResult;
          }
        }

        if (context.method === "next") {
          // Setting context._sent for legacy support of Babel's
          // function.sent implementation.
          context.sent = context._sent = context.arg;

        } else if (context.method === "throw") {
          if (state === GenStateSuspendedStart) {
            state = GenStateCompleted;
            throw context.arg;
          }

          context.dispatchException(context.arg);

        } else if (context.method === "return") {
          context.abrupt("return", context.arg);
        }

        state = GenStateExecuting;

        var record = tryCatch(innerFn, self, context);
        if (record.type === "normal") {
          // If an exception is thrown from innerFn, we leave state ===
          // GenStateExecuting and loop back for another invocation.
          state = context.done
            ? GenStateCompleted
            : GenStateSuspendedYield;

          if (record.arg === ContinueSentinel) {
            continue;
          }

          return {
            value: record.arg,
            done: context.done
          };

        } else if (record.type === "throw") {
          state = GenStateCompleted;
          // Dispatch the exception by looping back around to the
          // context.dispatchException(context.arg) call above.
          context.method = "throw";
          context.arg = record.arg;
        }
      }
    };
  }

  // Call delegate.iterator[context.method](context.arg) and handle the
  // result, either by returning a { value, done } result from the
  // delegate iterator, or by modifying context.method and context.arg,
  // setting context.delegate to null, and returning the ContinueSentinel.
  function maybeInvokeDelegate(delegate, context) {
    var method = delegate.iterator[context.method];
    if (method === undefined$1) {
      // A .throw or .return when the delegate iterator has no .throw
      // method always terminates the yield* loop.
      context.delegate = null;

      if (context.method === "throw") {
        // Note: ["return"] must be used for ES3 parsing compatibility.
        if (delegate.iterator["return"]) {
          // If the delegate iterator has a return method, give it a
          // chance to clean up.
          context.method = "return";
          context.arg = undefined$1;
          maybeInvokeDelegate(delegate, context);

          if (context.method === "throw") {
            // If maybeInvokeDelegate(context) changed context.method from
            // "return" to "throw", let that override the TypeError below.
            return ContinueSentinel;
          }
        }

        context.method = "throw";
        context.arg = new TypeError(
          "The iterator does not provide a 'throw' method");
      }

      return ContinueSentinel;
    }

    var record = tryCatch(method, delegate.iterator, context.arg);

    if (record.type === "throw") {
      context.method = "throw";
      context.arg = record.arg;
      context.delegate = null;
      return ContinueSentinel;
    }

    var info = record.arg;

    if (! info) {
      context.method = "throw";
      context.arg = new TypeError("iterator result is not an object");
      context.delegate = null;
      return ContinueSentinel;
    }

    if (info.done) {
      // Assign the result of the finished delegate to the temporary
      // variable specified by delegate.resultName (see delegateYield).
      context[delegate.resultName] = info.value;

      // Resume execution at the desired location (see delegateYield).
      context.next = delegate.nextLoc;

      // If context.method was "throw" but the delegate handled the
      // exception, let the outer generator proceed normally. If
      // context.method was "next", forget context.arg since it has been
      // "consumed" by the delegate iterator. If context.method was
      // "return", allow the original .return call to continue in the
      // outer generator.
      if (context.method !== "return") {
        context.method = "next";
        context.arg = undefined$1;
      }

    } else {
      // Re-yield the result returned by the delegate method.
      return info;
    }

    // The delegate iterator is finished, so forget it and continue with
    // the outer generator.
    context.delegate = null;
    return ContinueSentinel;
  }

  // Define Generator.prototype.{next,throw,return} in terms of the
  // unified ._invoke helper method.
  defineIteratorMethods(Gp);

  define(Gp, toStringTagSymbol, "Generator");

  // A Generator should always return itself as the iterator object when the
  // @@iterator function is called on it. Some browsers' implementations of the
  // iterator prototype chain incorrectly implement this, causing the Generator
  // object to not be returned from this call. This ensures that doesn't happen.
  // See https://github.com/facebook/regenerator/issues/274 for more details.
  Gp[iteratorSymbol] = function() {
    return this;
  };

  Gp.toString = function() {
    return "[object Generator]";
  };

  function pushTryEntry(locs) {
    var entry = { tryLoc: locs[0] };

    if (1 in locs) {
      entry.catchLoc = locs[1];
    }

    if (2 in locs) {
      entry.finallyLoc = locs[2];
      entry.afterLoc = locs[3];
    }

    this.tryEntries.push(entry);
  }

  function resetTryEntry(entry) {
    var record = entry.completion || {};
    record.type = "normal";
    delete record.arg;
    entry.completion = record;
  }

  function Context(tryLocsList) {
    // The root entry object (effectively a try statement without a catch
    // or a finally block) gives us a place to store values thrown from
    // locations where there is no enclosing try statement.
    this.tryEntries = [{ tryLoc: "root" }];
    tryLocsList.forEach(pushTryEntry, this);
    this.reset(true);
  }

  exports.keys = function(object) {
    var keys = [];
    for (var key in object) {
      keys.push(key);
    }
    keys.reverse();

    // Rather than returning an object with a next method, we keep
    // things simple and return the next function itself.
    return function next() {
      while (keys.length) {
        var key = keys.pop();
        if (key in object) {
          next.value = key;
          next.done = false;
          return next;
        }
      }

      // To avoid creating an additional object, we just hang the .value
      // and .done properties off the next function object itself. This
      // also ensures that the minifier will not anonymize the function.
      next.done = true;
      return next;
    };
  };

  function values(iterable) {
    if (iterable) {
      var iteratorMethod = iterable[iteratorSymbol];
      if (iteratorMethod) {
        return iteratorMethod.call(iterable);
      }

      if (typeof iterable.next === "function") {
        return iterable;
      }

      if (!isNaN(iterable.length)) {
        var i = -1, next = function next() {
          while (++i < iterable.length) {
            if (hasOwn.call(iterable, i)) {
              next.value = iterable[i];
              next.done = false;
              return next;
            }
          }

          next.value = undefined$1;
          next.done = true;

          return next;
        };

        return next.next = next;
      }
    }

    // Return an iterator with no values.
    return { next: doneResult };
  }
  exports.values = values;

  function doneResult() {
    return { value: undefined$1, done: true };
  }

  Context.prototype = {
    constructor: Context,

    reset: function(skipTempReset) {
      this.prev = 0;
      this.next = 0;
      // Resetting context._sent for legacy support of Babel's
      // function.sent implementation.
      this.sent = this._sent = undefined$1;
      this.done = false;
      this.delegate = null;

      this.method = "next";
      this.arg = undefined$1;

      this.tryEntries.forEach(resetTryEntry);

      if (!skipTempReset) {
        for (var name in this) {
          // Not sure about the optimal order of these conditions:
          if (name.charAt(0) === "t" &&
              hasOwn.call(this, name) &&
              !isNaN(+name.slice(1))) {
            this[name] = undefined$1;
          }
        }
      }
    },

    stop: function() {
      this.done = true;

      var rootEntry = this.tryEntries[0];
      var rootRecord = rootEntry.completion;
      if (rootRecord.type === "throw") {
        throw rootRecord.arg;
      }

      return this.rval;
    },

    dispatchException: function(exception) {
      if (this.done) {
        throw exception;
      }

      var context = this;
      function handle(loc, caught) {
        record.type = "throw";
        record.arg = exception;
        context.next = loc;

        if (caught) {
          // If the dispatched exception was caught by a catch block,
          // then let that catch block handle the exception normally.
          context.method = "next";
          context.arg = undefined$1;
        }

        return !! caught;
      }

      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        var record = entry.completion;

        if (entry.tryLoc === "root") {
          // Exception thrown outside of any try block that could handle
          // it, so set the completion value of the entire function to
          // throw the exception.
          return handle("end");
        }

        if (entry.tryLoc <= this.prev) {
          var hasCatch = hasOwn.call(entry, "catchLoc");
          var hasFinally = hasOwn.call(entry, "finallyLoc");

          if (hasCatch && hasFinally) {
            if (this.prev < entry.catchLoc) {
              return handle(entry.catchLoc, true);
            } else if (this.prev < entry.finallyLoc) {
              return handle(entry.finallyLoc);
            }

          } else if (hasCatch) {
            if (this.prev < entry.catchLoc) {
              return handle(entry.catchLoc, true);
            }

          } else if (hasFinally) {
            if (this.prev < entry.finallyLoc) {
              return handle(entry.finallyLoc);
            }

          } else {
            throw new Error("try statement without catch or finally");
          }
        }
      }
    },

    abrupt: function(type, arg) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.tryLoc <= this.prev &&
            hasOwn.call(entry, "finallyLoc") &&
            this.prev < entry.finallyLoc) {
          var finallyEntry = entry;
          break;
        }
      }

      if (finallyEntry &&
          (type === "break" ||
           type === "continue") &&
          finallyEntry.tryLoc <= arg &&
          arg <= finallyEntry.finallyLoc) {
        // Ignore the finally entry if control is not jumping to a
        // location outside the try/catch block.
        finallyEntry = null;
      }

      var record = finallyEntry ? finallyEntry.completion : {};
      record.type = type;
      record.arg = arg;

      if (finallyEntry) {
        this.method = "next";
        this.next = finallyEntry.finallyLoc;
        return ContinueSentinel;
      }

      return this.complete(record);
    },

    complete: function(record, afterLoc) {
      if (record.type === "throw") {
        throw record.arg;
      }

      if (record.type === "break" ||
          record.type === "continue") {
        this.next = record.arg;
      } else if (record.type === "return") {
        this.rval = this.arg = record.arg;
        this.method = "return";
        this.next = "end";
      } else if (record.type === "normal" && afterLoc) {
        this.next = afterLoc;
      }

      return ContinueSentinel;
    },

    finish: function(finallyLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.finallyLoc === finallyLoc) {
          this.complete(entry.completion, entry.afterLoc);
          resetTryEntry(entry);
          return ContinueSentinel;
        }
      }
    },

    "catch": function(tryLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.tryLoc === tryLoc) {
          var record = entry.completion;
          if (record.type === "throw") {
            var thrown = record.arg;
            resetTryEntry(entry);
          }
          return thrown;
        }
      }

      // The context.catch method must only be called with a location
      // argument that corresponds to a known catch block.
      throw new Error("illegal catch attempt");
    },

    delegateYield: function(iterable, resultName, nextLoc) {
      this.delegate = {
        iterator: values(iterable),
        resultName: resultName,
        nextLoc: nextLoc
      };

      if (this.method === "next") {
        // Deliberately forget the last sent value so that we don't
        // accidentally pass it on to the delegate.
        this.arg = undefined$1;
      }

      return ContinueSentinel;
    }
  };

  // Regardless of whether this script is executing as a CommonJS module
  // or not, return the runtime object so that we can declare the variable
  // regeneratorRuntime in the outer scope, which allows this module to be
  // injected easily by `bin/regenerator --include-runtime script.js`.
  return exports;

}(
  // If this script is executing as a CommonJS module, use module.exports
  // as the regeneratorRuntime namespace. Otherwise create a new empty
  // object. Either way, the resulting object will be used to initialize
  // the regeneratorRuntime variable at the top of this file.
   module.exports 
));

try {
  regeneratorRuntime = runtime;
} catch (accidentalStrictMode) {
  // This module should not be running in strict mode, so the above
  // assignment should always work unless something is misconfigured. Just
  // in case runtime.js accidentally runs in strict mode, we can escape
  // strict mode using a global Function call. This could conceivably fail
  // if a Content Security Policy forbids using Function, but in that case
  // the proper solution is to fix the accidental strict mode problem. If
  // you've misconfigured your bundler to force strict mode and applied a
  // CSP to forbid Function, and you're not willing to fix either of those
  // problems, please detail your unique predicament in a GitHub issue.
  Function("r", "regeneratorRuntime = r")(runtime);
}
});

function isSubtleCryptoAvailable() {
  return typeof crypto !== 'undefined' && typeof crypto.subtle !== 'undefined';
}
function isNodeCryptoAvailable(withFeature) {
  try {
    var resolvedResult = require.resolve('crypto');

    if (!resolvedResult) {
      return false;
    } // eslint-disable-next-line import/no-nodejs-modules,no-restricted-modules,global-require


    var cryptoModule = require('crypto');

    if (!cryptoModule) {
      return false;
    }

    if (withFeature) {
      var features = withFeature(cryptoModule);
      return features;
    }

    return true;
  } catch (error) {
    return false;
  }
}
var NO_CRYPTO_LIB = 'Crypto lib not found. Either the WebCrypto "crypto.subtle" or Node.js "crypto" module must be available.'; // Make async for future version which may lazy load.
// eslint-disable-next-line @typescript-eslint/require-await

function getCryptoLib() {
  return _getCryptoLib.apply(this, arguments);
}

function _getCryptoLib() {
  _getCryptoLib = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee() {
    var nodeCrypto;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (!isSubtleCryptoAvailable()) {
              _context.next = 4;
              break;
            }

            return _context.abrupt("return", {
              lib: crypto,
              name: 'webCrypto'
            });

          case 4:
            _context.prev = 4;
            // eslint-disable-next-line max-len
            // eslint-disable-next-line import/no-nodejs-modules,no-restricted-modules,global-require,@typescript-eslint/no-var-requires
            nodeCrypto = require('crypto');
            return _context.abrupt("return", {
              lib: nodeCrypto,
              name: 'nodeCrypto'
            });

          case 9:
            _context.prev = 9;
            _context.t0 = _context["catch"](4);
            throw new Error(NO_CRYPTO_LIB);

          case 12:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[4, 9]]);
  }));
  return _getCryptoLib.apply(this, arguments);
}

var NodeCryptoAesCipher = /*#__PURE__*/function () {
  function NodeCryptoAesCipher(createCipher, createDecipher) {
    this.createCipher = createCipher;
    this.createDecipher = createDecipher;
  }

  var _proto = NodeCryptoAesCipher.prototype;

  _proto.encrypt = /*#__PURE__*/function () {
    var _encrypt = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(algorithm, key, iv, data) {
      var cipher, result;
      return runtime_1.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (!(algorithm !== 'aes-128-cbc' && algorithm !== 'aes-256-cbc')) {
                _context.next = 2;
                break;
              }

              throw new Error("Unsupported cipher algorithm \"" + algorithm + "\"");

            case 2:
              cipher = this.createCipher(algorithm, key, iv);
              result = Buffer.concat([cipher.update(data), cipher["final"]()]);
              return _context.abrupt("return", Promise.resolve(result));

            case 5:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function encrypt(_x, _x2, _x3, _x4) {
      return _encrypt.apply(this, arguments);
    }

    return encrypt;
  }();

  _proto.decrypt = /*#__PURE__*/function () {
    var _decrypt = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(algorithm, key, iv, data) {
      var cipher, result;
      return runtime_1.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              if (!(algorithm !== 'aes-128-cbc' && algorithm !== 'aes-256-cbc')) {
                _context2.next = 2;
                break;
              }

              throw new Error("Unsupported cipher algorithm \"" + algorithm + "\"");

            case 2:
              cipher = this.createDecipher(algorithm, key, iv);
              result = Buffer.concat([cipher.update(data), cipher["final"]()]);
              return _context2.abrupt("return", Promise.resolve(result));

            case 5:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    function decrypt(_x5, _x6, _x7, _x8) {
      return _decrypt.apply(this, arguments);
    }

    return decrypt;
  }();

  return NodeCryptoAesCipher;
}();
var WebCryptoAesCipher = /*#__PURE__*/function () {
  function WebCryptoAesCipher(webCrypto) {
    this.webCrypto = webCrypto;
  }

  var _proto2 = WebCryptoAesCipher.prototype;

  _proto2.encrypt = /*#__PURE__*/function () {
    var _encrypt2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee3(algorithm, key, iv, data) {
      var algo, length, cryptoKey, result;
      return runtime_1.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              if (!(algorithm === 'aes-128-cbc')) {
                _context3.next = 5;
                break;
              }

              algo = 'AES-CBC';
              length = 128;
              _context3.next = 11;
              break;

            case 5:
              if (!(algorithm === 'aes-256-cbc')) {
                _context3.next = 10;
                break;
              }

              algo = 'AES-CBC';
              length = 256;
              _context3.next = 11;
              break;

            case 10:
              throw new Error("Unsupported cipher algorithm \"" + algorithm + "\"");

            case 11:
              _context3.next = 13;
              return this.webCrypto.subtle.importKey('raw', key, {
                name: algo,
                length: length
              }, false, ['encrypt']);

            case 13:
              cryptoKey = _context3.sent;
              _context3.next = 16;
              return this.webCrypto.subtle.encrypt({
                name: algo,
                iv: iv
              }, cryptoKey, data);

            case 16:
              result = _context3.sent;
              return _context3.abrupt("return", Buffer.from(result));

            case 18:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3, this);
    }));

    function encrypt(_x9, _x10, _x11, _x12) {
      return _encrypt2.apply(this, arguments);
    }

    return encrypt;
  }();

  _proto2.decrypt = /*#__PURE__*/function () {
    var _decrypt2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee4(algorithm, key, iv, data) {
      var algo, length, cryptoKey, result;
      return runtime_1.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              if (!(algorithm === 'aes-128-cbc')) {
                _context4.next = 5;
                break;
              }

              algo = 'AES-CBC';
              length = 128;
              _context4.next = 11;
              break;

            case 5:
              if (!(algorithm === 'aes-256-cbc')) {
                _context4.next = 10;
                break;
              }

              algo = 'AES-CBC';
              length = 256;
              _context4.next = 11;
              break;

            case 10:
              throw new Error("Unsupported cipher algorithm \"" + algorithm + "\"");

            case 11:
              _context4.next = 13;
              return this.webCrypto.subtle.importKey('raw', key, {
                name: algo,
                length: length
              }, false, ['decrypt']);

            case 13:
              cryptoKey = _context4.sent;
              _context4.next = 16;
              return this.webCrypto.subtle.decrypt({
                name: algo,
                iv: iv
              }, cryptoKey, data);

            case 16:
              result = _context4.sent;
              return _context4.abrupt("return", Buffer.from(result));

            case 18:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4, this);
    }));

    function decrypt(_x13, _x14, _x15, _x16) {
      return _decrypt2.apply(this, arguments);
    }

    return decrypt;
  }();

  return WebCryptoAesCipher;
}();
function createCipher() {
  return _createCipher.apply(this, arguments);
}

function _createCipher() {
  _createCipher = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee5() {
    var cryptoLib;
    return runtime_1.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return getCryptoLib();

          case 2:
            cryptoLib = _context5.sent;

            if (!(cryptoLib.name === 'webCrypto')) {
              _context5.next = 7;
              break;
            }

            return _context5.abrupt("return", new WebCryptoAesCipher(cryptoLib.lib));

          case 7:
            return _context5.abrupt("return", new NodeCryptoAesCipher(cryptoLib.lib.createCipheriv, cryptoLib.lib.createDecipheriv));

          case 8:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5);
  }));
  return _createCipher.apply(this, arguments);
}

var NodeCryptoHmacSha256 = /*#__PURE__*/function () {
  function NodeCryptoHmacSha256(createHmac) {
    this.createHmac = createHmac;
  }

  var _proto = NodeCryptoHmacSha256.prototype;

  _proto.digest = /*#__PURE__*/function () {
    var _digest = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(key, data) {
      var result;
      return runtime_1.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              result = this.createHmac('sha256', key).update(data).digest();
              return _context.abrupt("return", Promise.resolve(result));

            case 2:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function digest(_x, _x2) {
      return _digest.apply(this, arguments);
    }

    return digest;
  }();

  return NodeCryptoHmacSha256;
}();
var WebCryptoHmacSha256 = /*#__PURE__*/function () {
  function WebCryptoHmacSha256(webCrypto) {
    this.webCrypto = webCrypto;
  }

  var _proto2 = WebCryptoHmacSha256.prototype;

  _proto2.digest = /*#__PURE__*/function () {
    var _digest2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(key, data) {
      var cryptoKey, sig;
      return runtime_1.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return this.webCrypto.subtle.importKey('raw', key, {
                name: 'HMAC',
                hash: 'SHA-256'
              }, true, ['sign']);

            case 2:
              cryptoKey = _context2.sent;
              _context2.next = 5;
              return this.webCrypto.subtle.sign( // The `hash` is only specified for non-compliant browsers like Edge.
              {
                name: 'HMAC',
                hash: 'SHA-256'
              }, cryptoKey, data);

            case 5:
              sig = _context2.sent;
              return _context2.abrupt("return", Buffer.from(sig));

            case 7:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    function digest(_x3, _x4) {
      return _digest2.apply(this, arguments);
    }

    return digest;
  }();

  return WebCryptoHmacSha256;
}();
function createHmacSha256() {
  return _createHmacSha.apply(this, arguments);
}

function _createHmacSha() {
  _createHmacSha = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee3() {
    var cryptoLib;
    return runtime_1.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return getCryptoLib();

          case 2:
            cryptoLib = _context3.sent;

            if (!(cryptoLib.name === 'webCrypto')) {
              _context3.next = 7;
              break;
            }

            return _context3.abrupt("return", new WebCryptoHmacSha256(cryptoLib.lib));

          case 7:
            return _context3.abrupt("return", new NodeCryptoHmacSha256(cryptoLib.lib.createHmac));

          case 8:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));
  return _createHmacSha.apply(this, arguments);
}

var NodeCryptoSha2Hash = /*#__PURE__*/function () {
  function NodeCryptoSha2Hash(createHash) {
    this.createHash = createHash;
  }

  var _proto = NodeCryptoSha2Hash.prototype;

  _proto.digest = /*#__PURE__*/function () {
    var _digest = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(data, algorithm) {
      var result;
      return runtime_1.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (algorithm === void 0) {
                algorithm = 'sha256';
              }

              _context.prev = 1;
              result = this.createHash(algorithm).update(data).digest();
              return _context.abrupt("return", Promise.resolve(result));

            case 6:
              _context.prev = 6;
              _context.t0 = _context["catch"](1);
              console.error(_context.t0);
              console.error("Error performing " + algorithm + " digest with Node.js 'crypto.createHash'");
              throw _context.t0;

            case 11:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this, [[1, 6]]);
    }));

    function digest(_x, _x2) {
      return _digest.apply(this, arguments);
    }

    return digest;
  }();

  return NodeCryptoSha2Hash;
}();
var WebCryptoSha2Hash = /*#__PURE__*/function () {
  function WebCryptoSha2Hash(webCrypto) {
    this.webCrypto = webCrypto;
  }

  var _proto2 = WebCryptoSha2Hash.prototype;

  _proto2.digest = /*#__PURE__*/function () {
    var _digest2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(data, algorithm) {
      var algo, hash;
      return runtime_1.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              if (algorithm === void 0) {
                algorithm = 'sha256';
              }

              if (!(algorithm === 'sha256')) {
                _context2.next = 5;
                break;
              }

              algo = 'SHA-256';
              _context2.next = 10;
              break;

            case 5:
              if (!(algorithm === 'sha512')) {
                _context2.next = 9;
                break;
              }

              algo = 'SHA-512';
              _context2.next = 10;
              break;

            case 9:
              throw new Error("Unsupported hash algorithm " + algorithm);

            case 10:
              _context2.prev = 10;
              _context2.next = 13;
              return this.webCrypto.subtle.digest(algo, data);

            case 13:
              hash = _context2.sent;
              return _context2.abrupt("return", Buffer.from(hash));

            case 17:
              _context2.prev = 17;
              _context2.t0 = _context2["catch"](10);
              console.error(_context2.t0);
              console.error("Error performing " + algorithm + " digest with WebCrypto.");
              throw _context2.t0;

            case 22:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this, [[10, 17]]);
    }));

    function digest(_x3, _x4) {
      return _digest2.apply(this, arguments);
    }

    return digest;
  }();

  return WebCryptoSha2Hash;
}();
function createSha2Hash() {
  return _createSha2Hash.apply(this, arguments);
}

function _createSha2Hash() {
  _createSha2Hash = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee3() {
    var cryptoLib;
    return runtime_1.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return getCryptoLib();

          case 2:
            cryptoLib = _context3.sent;

            if (!(cryptoLib.name === 'webCrypto')) {
              _context3.next = 7;
              break;
            }

            return _context3.abrupt("return", new WebCryptoSha2Hash(cryptoLib.lib));

          case 7:
            return _context3.abrupt("return", new NodeCryptoSha2Hash(cryptoLib.lib.createHash));

          case 8:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));
  return _createSha2Hash.apply(this, arguments);
}

function hashSha256(_x5) {
  return _hashSha.apply(this, arguments);
}

function _hashSha() {
  _hashSha = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee4(data) {
    var hash;
    return runtime_1.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return createSha2Hash();

          case 2:
            hash = _context4.sent;
            _context4.next = 5;
            return hash.digest(data, 'sha256');

          case 5:
            return _context4.abrupt("return", _context4.sent);

          case 6:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));
  return _hashSha.apply(this, arguments);
}

function hashSha512(_x6) {
  return _hashSha2.apply(this, arguments);
}

function _hashSha2() {
  _hashSha2 = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee5(data) {
    var hash;
    return runtime_1.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return createSha2Hash();

          case 2:
            hash = _context5.sent;
            _context5.next = 5;
            return hash.digest(data, 'sha512');

          case 5:
            return _context5.abrupt("return", _context5.sent);

          case 6:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5);
  }));
  return _hashSha2.apply(this, arguments);
}

function aes256CbcEncrypt(_x, _x2, _x3) {
  return _aes256CbcEncrypt.apply(this, arguments);
}

function _aes256CbcEncrypt() {
  _aes256CbcEncrypt = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(iv, key, plaintext) {
    var cipher, result;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return createCipher();

          case 2:
            cipher = _context.sent;
            _context.next = 5;
            return cipher.encrypt('aes-256-cbc', key, iv, plaintext);

          case 5:
            result = _context.sent;
            return _context.abrupt("return", result);

          case 7:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _aes256CbcEncrypt.apply(this, arguments);
}

function hmacSha256(_x4, _x5) {
  return _hmacSha.apply(this, arguments);
}

function _hmacSha() {
  _hmacSha = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(key, content) {
    var hmacSha256;
    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return createHmacSha256();

          case 2:
            hmacSha256 = _context2.sent;
            return _context2.abrupt("return", hmacSha256.digest(key, content));

          case 4:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return _hmacSha.apply(this, arguments);
}

function sharedSecretToKeys(_x6) {
  return _sharedSecretToKeys.apply(this, arguments);
}

function _sharedSecretToKeys() {
  _sharedSecretToKeys = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee3(sharedSecret) {
    var hashedSecret;
    return runtime_1.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return hashSha512(sharedSecret);

          case 2:
            hashedSecret = _context3.sent;
            return _context3.abrupt("return", {
              encryptionKey: hashedSecret.slice(0, 32),
              hmacKey: hashedSecret.slice(32)
            });

          case 4:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));
  return _sharedSecretToKeys.apply(this, arguments);
}

function equalConstTime(b1, b2) {
  if (b1.length !== b2.length) {
    return false;
  }

  var res = 0;

  for (var i = 0; i < b1.length; i++) {
    res |= b1[i] ^ b2[i]; // jshint ignore:line
  }

  return res === 0;
}
function aes256CbcDecrypt(_x7, _x8, _x9) {
  return _aes256CbcDecrypt.apply(this, arguments);
}

function _aes256CbcDecrypt() {
  _aes256CbcDecrypt = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee4(iv, key, ciphertext) {
    var cipher, result;
    return runtime_1.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return createCipher();

          case 2:
            cipher = _context4.sent;
            _context4.next = 5;
            return cipher.decrypt('aes-256-cbc', key, iv, ciphertext);

          case 5:
            result = _context4.sent;
            return _context4.abrupt("return", result);

          case 7:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));
  return _aes256CbcDecrypt.apply(this, arguments);
}

// Extracted from https://github.com/crypto-browserify/ripemd160
var ARRAY16 = /*#__PURE__*/new Array(16);
var zl = /*#__PURE__*/initU8Array([0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 7, 4, 13, 1, 10, 6, 15, 3, 12, 0, 9, 5, 2, 14, 11, 8, 3, 10, 14, 4, 9, 15, 8, 1, 2, 7, 0, 6, 13, 11, 5, 12, 1, 9, 11, 10, 0, 8, 12, 4, 13, 3, 7, 15, 14, 5, 6, 2, 4, 0, 5, 9, 7, 12, 2, 10, 14, 1, 3, 8, 11, 6, 15, 13]);
var zr = /*#__PURE__*/initU8Array([5, 14, 7, 0, 9, 2, 11, 4, 13, 6, 15, 8, 1, 10, 3, 12, 6, 11, 3, 7, 0, 13, 5, 10, 14, 15, 8, 12, 4, 9, 1, 2, 15, 5, 1, 3, 7, 14, 6, 9, 11, 8, 12, 2, 10, 0, 4, 13, 8, 6, 4, 1, 3, 11, 15, 0, 5, 12, 2, 13, 9, 7, 10, 14, 12, 15, 10, 4, 1, 5, 8, 7, 6, 2, 13, 14, 0, 3, 9, 11]);
var sl = /*#__PURE__*/initU8Array([11, 14, 15, 12, 5, 8, 7, 9, 11, 13, 14, 15, 6, 7, 9, 8, 7, 6, 8, 13, 11, 9, 7, 15, 7, 12, 15, 9, 11, 7, 13, 12, 11, 13, 6, 7, 14, 9, 13, 15, 14, 8, 13, 6, 5, 12, 7, 5, 11, 12, 14, 15, 14, 15, 9, 8, 9, 14, 5, 6, 8, 6, 5, 12, 9, 15, 5, 11, 6, 8, 13, 12, 5, 12, 13, 14, 11, 8, 5, 6]);
var sr = /*#__PURE__*/initU8Array([8, 9, 9, 11, 13, 15, 15, 5, 7, 7, 8, 11, 14, 14, 12, 6, 9, 13, 15, 7, 12, 8, 9, 11, 7, 7, 12, 7, 6, 15, 13, 11, 9, 7, 15, 11, 8, 6, 6, 14, 12, 13, 5, 14, 13, 13, 7, 5, 15, 5, 8, 11, 14, 14, 6, 14, 6, 9, 12, 9, 12, 5, 15, 8, 8, 5, 12, 9, 12, 5, 14, 6, 8, 13, 6, 5, 15, 13, 11, 11]);
var hl = /*#__PURE__*/initU32Array([0x00000000, 0x5a827999, 0x6ed9eba1, 0x8f1bbcdc, 0xa953fd4e]);
var hr = /*#__PURE__*/initU32Array([0x50a28be6, 0x5c4dd124, 0x6d703ef3, 0x7a6d76e9, 0x00000000]);

function rotl(x, n) {
  return x << n | x >>> 32 - n;
}

function fn1(a, b, c, d, e, m, k, s) {
  return rotl(a + (b ^ c ^ d) + m + k | 0, s) + e | 0;
}

function fn2(a, b, c, d, e, m, k, s) {
  return rotl(a + (b & c | ~b & d) + m + k | 0, s) + e | 0;
}

function fn3(a, b, c, d, e, m, k, s) {
  return rotl(a + ((b | ~c) ^ d) + m + k | 0, s) + e | 0;
}

function fn4(a, b, c, d, e, m, k, s) {
  return rotl(a + (b & d | c & ~d) + m + k | 0, s) + e | 0;
}

function fn5(a, b, c, d, e, m, k, s) {
  return rotl(a + (b ^ (c | ~d)) + m + k | 0, s) + e | 0;
}

function readInt32LE(buffer, offset) {
  offset >>>= 0;
  return buffer[offset] | buffer[offset + 1] << 8 | buffer[offset + 2] << 16 | buffer[offset + 3] << 24;
}

function writeUInt32LE(buffer, value, offset) {
  value = +value;
  offset >>>= 0;
  buffer[offset + 3] = value >>> 24;
  buffer[offset + 2] = value >>> 16;
  buffer[offset + 1] = value >>> 8;
  buffer[offset] = value & 0xff;
  return offset + 4;
}

function writeInt32LE(buffer, value, offset) {
  value = +value;
  offset >>>= 0;
  buffer[offset] = value & 0xff;
  buffer[offset + 1] = value >>> 8;
  buffer[offset + 2] = value >>> 16;
  buffer[offset + 3] = value >>> 24;
  return offset + 4;
}

function initU32Array(data) {
  if (typeof Uint32Array !== 'undefined') {
    return new Uint32Array(data);
  } else {
    return data;
  }
}

function initU8Array(data) {
  if (typeof Uint8Array !== 'undefined') {
    return new Uint8Array(data);
  } else {
    return data;
  }
}

function createU8Array(size) {
  if (typeof Uint8Array !== 'undefined') {
    return new Uint8Array(size);
  } else {
    return new Array(size);
  }
}

var RIPEMD160 = /*#__PURE__*/function () {
  function RIPEMD160() {
    this._block = createU8Array(64);
    this._blockSize = 64;
    this._blockOffset = 0;
    this._length = [0, 0, 0, 0];
    this._finalized = false;
    this._a = 0x67452301;
    this._b = 0xefcdab89;
    this._c = 0x98badcfe;
    this._d = 0x10325476;
    this._e = 0xc3d2e1f0;
  }

  var _proto = RIPEMD160.prototype;

  _proto.update = function update(data) {
    if (this._finalized) throw new Error('Digest already called'); // consume data

    var block = this._block;
    var offset = 0;

    while (this._blockOffset + data.length - offset >= this._blockSize) {
      for (var i = this._blockOffset; i < this._blockSize;) {
        block[i++] = data[offset++];
      }

      this._update();

      this._blockOffset = 0;
    }

    while (offset < data.length) {
      block[this._blockOffset++] = data[offset++];
    } // update length


    for (var j = 0, carry = data.length * 8; carry > 0; ++j) {
      this._length[j] += carry;
      carry = this._length[j] / 0x0100000000 | 0;
      if (carry > 0) this._length[j] -= 0x0100000000 * carry;
    }

    return this;
  };

  _proto._update = function _update() {
    var words = ARRAY16;

    for (var j = 0; j < 16; ++j) {
      words[j] = readInt32LE(this._block, j * 4);
    }

    var al = this._a | 0;
    var bl = this._b | 0;
    var cl = this._c | 0;
    var dl = this._d | 0;
    var el = this._e | 0;
    var ar = this._a | 0;
    var br = this._b | 0;
    var cr = this._c | 0;
    var dr = this._d | 0;
    var er = this._e | 0; // computation

    for (var i = 0; i < 80; i += 1) {
      var tl = void 0;
      var tr = void 0;

      if (i < 16) {
        tl = fn1(al, bl, cl, dl, el, words[zl[i]], hl[0], sl[i]);
        tr = fn5(ar, br, cr, dr, er, words[zr[i]], hr[0], sr[i]);
      } else if (i < 32) {
        tl = fn2(al, bl, cl, dl, el, words[zl[i]], hl[1], sl[i]);
        tr = fn4(ar, br, cr, dr, er, words[zr[i]], hr[1], sr[i]);
      } else if (i < 48) {
        tl = fn3(al, bl, cl, dl, el, words[zl[i]], hl[2], sl[i]);
        tr = fn3(ar, br, cr, dr, er, words[zr[i]], hr[2], sr[i]);
      } else if (i < 64) {
        tl = fn4(al, bl, cl, dl, el, words[zl[i]], hl[3], sl[i]);
        tr = fn2(ar, br, cr, dr, er, words[zr[i]], hr[3], sr[i]);
      } else {
        // if (i<80) {
        tl = fn5(al, bl, cl, dl, el, words[zl[i]], hl[4], sl[i]);
        tr = fn1(ar, br, cr, dr, er, words[zr[i]], hr[4], sr[i]);
      }

      al = el;
      el = dl;
      dl = rotl(cl, 10);
      cl = bl;
      bl = tl;
      ar = er;
      er = dr;
      dr = rotl(cr, 10);
      cr = br;
      br = tr;
    } // update state


    var t = this._b + cl + dr | 0;
    this._b = this._c + dl + er | 0;
    this._c = this._d + el + ar | 0;
    this._d = this._e + al + br | 0;
    this._e = this._a + bl + cr | 0;
    this._a = t;
  };

  _proto.digest = function digest() {
    if (this._finalized) {
      throw new Error('Digest already called');
    }

    this._finalized = true; // create padding and handle blocks

    this._block[this._blockOffset++] = 0x80;

    if (this._blockOffset > 56) {
      this._block.fill(0, this._blockOffset, 64);

      this._update();

      this._blockOffset = 0;
    }

    this._block.fill(0, this._blockOffset, 56);

    writeUInt32LE(this._block, this._length[0], 56);
    writeUInt32LE(this._block, this._length[1], 60);

    this._update(); // produce result


    var buffer = createU8Array(20);
    writeInt32LE(buffer, this._a, 0);
    writeInt32LE(buffer, this._b, 4);
    writeInt32LE(buffer, this._c, 8);
    writeInt32LE(buffer, this._d, 12);
    writeInt32LE(buffer, this._e, 16); // reset state

    this._block.fill(0);

    this._blockOffset = 0;

    for (var i = 0; i < 4; ++i) {
      this._length[i] = 0;
    }

    return buffer;
  };

  return RIPEMD160;
}();

var Ripemd160PolyfillDigest = /*#__PURE__*/function () {
  function Ripemd160PolyfillDigest() {}

  var _proto = Ripemd160PolyfillDigest.prototype;

  _proto.digest = function digest(data) {
    var instance = new RIPEMD160();
    instance.update(data);
    var hash = instance.digest();

    if (Array.isArray(hash)) {
      return Buffer.from(hash);
    } else {
      return Buffer.from(hash.buffer);
    }
  };

  return Ripemd160PolyfillDigest;
}();
var NodeCryptoRipemd160Digest = /*#__PURE__*/function () {
  function NodeCryptoRipemd160Digest(nodeCryptoCreateHash) {
    this.nodeCryptoCreateHash = nodeCryptoCreateHash;
  }

  var _proto2 = NodeCryptoRipemd160Digest.prototype;

  _proto2.digest = function digest(data) {
    try {
      return this.nodeCryptoCreateHash('rmd160').update(data).digest();
    } catch (error) {
      try {
        return this.nodeCryptoCreateHash('ripemd160').update(data).digest();
      } catch (_err) {
        console.log(error);
        console.log('Node.js `crypto.createHash` exists but failing to digest for ripemd160, falling back to js implementation');
        var polyfill = new Ripemd160PolyfillDigest();
        return polyfill.digest(data);
      }
    }
  };

  return NodeCryptoRipemd160Digest;
}();
function createHashRipemd160() {
  var nodeCryptoCreateHash = isNodeCryptoAvailable(function (nodeCrypto) {
    if (typeof nodeCrypto.createHash === 'function') {
      return nodeCrypto.createHash;
    }

    return false;
  });

  if (nodeCryptoCreateHash) {
    return new NodeCryptoRipemd160Digest(nodeCryptoCreateHash);
  } else {
    return new Ripemd160PolyfillDigest();
  }
}
function hashRipemd160(data) {
  var hash = createHashRipemd160();
  return hash.digest(data);
}

function getRandomBytes(_x) {
  return _getRandomBytes.apply(this, arguments);
}

function _getRandomBytes() {
  _getRandomBytes = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(size) {
    var cryptoLib;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return getCryptoLib();

          case 2:
            cryptoLib = _context.sent;

            if (!(cryptoLib.name === 'webCrypto')) {
              _context.next = 7;
              break;
            }

            return _context.abrupt("return", cryptoLib.lib.getRandomValues(Buffer.alloc(size)));

          case 7:
            return _context.abrupt("return", cryptoLib.lib.randomBytes(size));

          case 8:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _getRandomBytes.apply(this, arguments);
}

function encryptECIES(_x) {
  return _encryptECIES.apply(this, arguments);
}

function _encryptECIES() {
  _encryptECIES = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(options) {
    var publicKey, content, cipherTextEncoding, wasString, ephemeralPrivateKey, ephemeralPublicKey, sharedSecret, sharedKeys, initializationVector, cipherText, macData, mac, cipherTextString, result;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            publicKey = options.publicKey, content = options.content, cipherTextEncoding = options.cipherTextEncoding, wasString = options.wasString;
            ephemeralPrivateKey = utils.randomPrivateKey();
            ephemeralPublicKey = getPublicKey(ephemeralPrivateKey, true);
            sharedSecret = getSharedSecret(ephemeralPrivateKey, publicKey, true); // Trim the compressed mode prefix byte

            sharedSecret = sharedSecret.slice(1);
            _context.next = 7;
            return sharedSecretToKeys(sharedSecret);

          case 7:
            sharedKeys = _context.sent;
            _context.next = 10;
            return getRandomBytes(16);

          case 10:
            initializationVector = _context.sent;
            _context.next = 13;
            return aes256CbcEncrypt(initializationVector, sharedKeys.encryptionKey, content);

          case 13:
            cipherText = _context.sent;
            macData = Buffer.concat([initializationVector, ephemeralPublicKey, cipherText]);
            _context.next = 17;
            return hmacSha256(sharedKeys.hmacKey, macData);

          case 17:
            mac = _context.sent;

            if (!(!cipherTextEncoding || cipherTextEncoding === 'hex')) {
              _context.next = 22;
              break;
            }

            cipherTextString = cipherText.toString('hex');
            _context.next = 27;
            break;

          case 22:
            if (!(cipherTextEncoding === 'base64')) {
              _context.next = 26;
              break;
            }

            cipherTextString = cipherText.toString('base64');
            _context.next = 27;
            break;

          case 26:
            throw new Error("Unexpected cipherTextEncoding \"" + cipherTextEncoding + "\"");

          case 27:
            result = {
              iv: initializationVector.toString('hex'),
              ephemeralPK: Buffer.from(ephemeralPublicKey.buffer).toString('hex'),
              cipherText: cipherTextString,
              mac: mac.toString('hex'),
              wasString: wasString
            };

            if (cipherTextEncoding && cipherTextEncoding !== 'hex') {
              result.cipherTextEncoding = cipherTextEncoding;
            }

            return _context.abrupt("return", result);

          case 30:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _encryptECIES.apply(this, arguments);
}

/**
 * Decrypt content encrypted using ECIES
 *  * @param options {DecryptECIESOptions}
 *  iv (initialization vector), cipherText (cipher text),
 *  mac (message authentication code), ephemeralPublicKey
 *  wasString (boolean indicating with or not to return a buffer or string on decrypt)
 * @return {Buffer} plaintext
 * @throws {Error} if unable to decrypt
 * @private
 * @ignore
 */

function decryptECIES(_x) {
  return _decryptECIES.apply(this, arguments);
}

function _decryptECIES() {
  _decryptECIES = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(options) {
    var privateKey, cipherObject, ephemeralPK, sharedSecret, sharedKeys, ivBuffer, cipherTextBuffer, macData, actualMac, expectedMac, plainText;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            privateKey = options.privateKey, cipherObject = options.cipherObject;
            ephemeralPK = cipherObject.ephemeralPK;
            sharedSecret = Buffer.from(getSharedSecret(privateKey, ephemeralPK, true), 'hex'); // Trim the compressed mode prefix byte

            sharedSecret = sharedSecret.slice(1);
            _context.next = 6;
            return sharedSecretToKeys(sharedSecret);

          case 6:
            sharedKeys = _context.sent;
            ivBuffer = Buffer.from(cipherObject.iv, 'hex');

            if (!(!cipherObject.cipherTextEncoding || cipherObject.cipherTextEncoding === 'hex')) {
              _context.next = 12;
              break;
            }

            cipherTextBuffer = Buffer.from(cipherObject.cipherText, 'hex');
            _context.next = 17;
            break;

          case 12:
            if (!(cipherObject.cipherTextEncoding === 'base64')) {
              _context.next = 16;
              break;
            }

            cipherTextBuffer = Buffer.from(cipherObject.cipherText, 'base64');
            _context.next = 17;
            break;

          case 16:
            throw new Error("Unexpected cipherTextEncoding \"" + cipherObject.cipherText + "\"");

          case 17:
            macData = Buffer.concat([ivBuffer, Buffer.from(ephemeralPK, 'hex'), cipherTextBuffer]);
            _context.next = 20;
            return hmacSha256(sharedKeys.hmacKey, macData);

          case 20:
            actualMac = _context.sent;
            expectedMac = Buffer.from(cipherObject.mac, 'hex');

            if (equalConstTime(expectedMac, actualMac)) {
              _context.next = 24;
              break;
            }

            throw new Error('Decryption failed: failure in MAC check');

          case 24:
            _context.next = 26;
            return aes256CbcDecrypt(ivBuffer, sharedKeys.encryptionKey, cipherTextBuffer);

          case 26:
            plainText = _context.sent;

            if (!cipherObject.wasString) {
              _context.next = 31;
              break;
            }

            return _context.abrupt("return", plainText.toString());

          case 31:
            return _context.abrupt("return", plainText);

          case 32:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _decryptECIES.apply(this, arguments);
}

var Versions;

(function (Versions) {
  Versions[Versions["mainnetP2SH"] = 0] = "mainnetP2SH";
  Versions[Versions["testnetP2SH"] = 111] = "testnetP2SH";
})(Versions || (Versions = {}));

var bitcoin = {
  messagePrefix: '\x18Bitcoin Signed Message:\n',
  bech32: 'bc',
  bip32: {
    "public": 0x0488b21e,
    "private": 0x0488ade4
  },
  pubKeyHash: 0x00,
  scriptHash: 0x05,
  wif: 0x80
};
var regtest = {
  messagePrefix: '\x18Bitcoin Signed Message:\n',
  bech32: 'bcrt',
  bip32: {
    "public": 0x043587cf,
    "private": 0x04358394
  },
  pubKeyHash: 0x6f,
  scriptHash: 0xc4,
  wif: 0xef
};
var testnet = {
  messagePrefix: '\x18Bitcoin Signed Message:\n',
  bech32: 'tb',
  bip32: {
    "public": 0x043587cf,
    "private": 0x04358394
  },
  pubKeyHash: 0x6f,
  scriptHash: 0xc4,
  wif: 0xef
};
var networks = {
  bitcoin: bitcoin,
  regtest: regtest,
  testnet: testnet
};

// function toBase58Check(hash, version) {
//   typeforce(types.tuple(types.Hash160bit, types.UInt8), arguments);
//   const payload = Buffer.allocUnsafe(21);
//   payload.writeUInt8(version, 0);
//   hash.copy(payload, 1);
//   return bs58check.encode(payload);
// }
// @see: https://github.com/bitcoinjs/bitcoinjs-lib/blob/master/src/address.js#L30
// https://github.com/paulmillr/micro-base58

function base58check(_x, _x2) {
  return _base58check.apply(this, arguments);
}

function _base58check() {
  _base58check = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(hash, version) {
    var versionBuffer, data, payload, sha1, sha2, checksum;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            versionBuffer = Buffer.alloc(1, version);
            data = new Uint8Array(25);
            payload = Buffer.allocUnsafe(21);
            payload.writeUInt8(version);
            hash.copy(payload, 1);
            _context.next = 7;
            return hashSha256(payload);

          case 7:
            sha1 = _context.sent;
            _context.next = 10;
            return hashSha256(sha1);

          case 10:
            sha2 = _context.sent;
            checksum = sha2.slice(0, 4);
            data.set(versionBuffer, 0);
            data.set(hash, 1);
            data.set(checksum, hash.length + 1);
            return _context.abrupt("return", base58(data));

          case 16:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _base58check.apply(this, arguments);
}

function privateKeyToAddress(_x) {
  return _privateKeyToAddress.apply(this, arguments);
}

function _privateKeyToAddress() {
  _privateKeyToAddress = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(privateKey) {
    var publicKey, sha256, hash160;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            publicKey = getPublicKey(privateKey, true); // is it compressed though?

            _context.next = 3;
            return hashSha256(Buffer.from(publicKey, 'hex'));

          case 3:
            sha256 = _context.sent;
            hash160 = hashRipemd160(sha256);
            return _context.abrupt("return", base58check(hash160, networks.bitcoin.pubKeyHash));

          case 6:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _privateKeyToAddress.apply(this, arguments);
}

function publicKeyToAddress(_x2) {
  return _publicKeyToAddress.apply(this, arguments);
}

function _publicKeyToAddress() {
  _publicKeyToAddress = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(publicKey) {
    var sha256, hash160;
    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return hashSha256(Buffer.from(publicKey, 'hex'));

          case 2:
            sha256 = _context2.sent;
            hash160 = hashRipemd160(sha256);
            return _context2.abrupt("return", base58check(hash160, networks.bitcoin.pubKeyHash));

          case 5:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return _publicKeyToAddress.apply(this, arguments);
}

// @ts-nocheck
function padString(input) {
  var segmentLength = 4;
  var stringLength = input.length;
  var diff = stringLength % segmentLength;

  if (!diff) {
    return input;
  }

  var position = stringLength;
  var padLength = segmentLength - diff;
  var paddedStringLength = stringLength + padLength;
  var buffer = Buffer.alloc(paddedStringLength);
  buffer.write(input);

  while (padLength--) {
    buffer.write('=', position++);
  }

  return buffer.toString();
}

function encode(input, encoding) {
  if (encoding === void 0) {
    encoding = 'utf8';
  }

  if (Buffer.isBuffer(input)) {
    return fromBase64(input.toString('base64'));
  }

  return fromBase64(Buffer.from(input, encoding).toString('base64'));
}

function decode(base64url, encoding) {
  if (encoding === void 0) {
    encoding = 'utf8';
  }

  return Buffer.from(toBase64(base64url), 'base64').toString(encoding);
}

function toBase64(base64url) {
  // We this to be a string so we can do .replace on it. If it's
  // already a string, this is a noop.
  base64url = base64url.toString();
  return padString(base64url).replace(/\-/g, '+').replace(/_/g, '/');
}

function fromBase64(base64) {
  return base64.replace(/=/g, '').replace(/\+/g, '-').replace(/\//g, '_');
}

function toBuffer(base64url) {
  return Buffer.from(toBase64(base64url), 'base64');
}

var base64url = encode;
base64url.encode = encode;
base64url.decode = decode;
base64url.toBase64 = toBase64;
base64url.fromBase64 = fromBase64;
base64url.toBuffer = toBuffer;

function decodeToken(token) {
  if (typeof token === 'string') {
    // decompose the token into parts
    var tokenParts = token.split('.');
    var header = JSON.parse(base64url.decode(tokenParts[0]));
    var payload = JSON.parse(base64url.decode(tokenParts[1]));
    var signature = tokenParts[2]; // return the token object

    return {
      header: header,
      payload: payload,
      signature: signature
    };
  } else if (typeof token === 'object') {
    if (typeof token.payload !== 'string') {
      throw new Error('Expected token payload to be a base64 or json string');
    }

    var _payload = token.payload;

    if (token.payload[0] !== '{') {
      _payload = base64url.decode(_payload);
    }

    var allHeaders = [];
    token.header.map(function (headerValue) {
      var header = JSON.parse(base64url.decode(headerValue));
      allHeaders.push(header);
    });
    return {
      header: allHeaders,
      payload: JSON.parse(_payload),
      signature: token.signature
    };
  }
}

function createSigningInput(payload, header) {
  var tokenParts = []; // add in the header

  var encodedHeader = base64url.encode(JSON.stringify(header));
  tokenParts.push(encodedHeader); // add in the payload

  var encodedPayload = base64url.encode(JSON.stringify(payload));
  tokenParts.push(encodedPayload); // prepare the message

  var signingInput = tokenParts.join('.'); // return the signing input

  return signingInput;
}

function createUnsecuredToken(payload) {
  var header = {
    typ: 'JWT',
    alg: 'none'
  };
  return createSigningInput(payload, header) + '.';
}
var TokenSigner = /*#__PURE__*/function () {
  function TokenSigner(signingAlgorithm, rawPrivateKey) {
    if (!(signingAlgorithm && rawPrivateKey)) {
      throw new Error('a signing algorithm and private key are required');
    }

    this.tokenType = 'JWT';
    this.rawPrivateKey = rawPrivateKey;
  }

  var _proto = TokenSigner.prototype;

  _proto.header = function header(_header) {
    if (_header === void 0) {
      _header = {};
    }

    var defaultHeader = {
      typ: this.tokenType,
      alg: 'ES256K'
    };
    return Object.assign({}, defaultHeader, _header);
  };

  _proto.sign = /*#__PURE__*/function () {
    var _sign = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(payload, expanded, customHeader) {
      var header, signingInput, signingInputHash;
      return runtime_1.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (expanded === void 0) {
                expanded = false;
              }

              if (customHeader === void 0) {
                customHeader = {};
              }

              // generate the token header
              header = this.header(customHeader); // prepare the message to be signed

              signingInput = createSigningInput(payload, header);
              _context.next = 6;
              return hashSha256(Buffer.from(signingInput));

            case 6:
              signingInputHash = _context.sent;
              return _context.abrupt("return", this.createWithSignedHash(payload, expanded, header, signingInput, signingInputHash));

            case 8:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function sign(_x, _x2, _x3) {
      return _sign.apply(this, arguments);
    }

    return sign;
  }();

  _proto.createWithSignedHash = /*#__PURE__*/function () {
    var _createWithSignedHash = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(payload, expanded, header, signingInput, signingInputHash) {
      var sig, sigBuf, formatted, signedToken;
      return runtime_1.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return sign(signingInputHash, this.rawPrivateKey);

            case 2:
              sig = _context2.sent;
              sigBuf = Buffer.from(sig);
              formatted = derToJose(sigBuf, 'ES256');

              if (!expanded) {
                _context2.next = 10;
                break;
              }

              signedToken = {
                header: [base64url.encode(JSON.stringify(header))],
                payload: JSON.stringify(payload),
                signature: [formatted]
              };
              return _context2.abrupt("return", signedToken);

            case 10:
              return _context2.abrupt("return", [signingInput, formatted].join('.'));

            case 11:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    function createWithSignedHash(_x4, _x5, _x6, _x7, _x8) {
      return _createWithSignedHash.apply(this, arguments);
    }

    return createWithSignedHash;
  }();

  return TokenSigner;
}();

export { NO_CRYPTO_LIB, NodeCryptoAesCipher, NodeCryptoHmacSha256, NodeCryptoRipemd160Digest, NodeCryptoSha2Hash, RIPEMD160, Ripemd160PolyfillDigest, TokenSigner, WebCryptoAesCipher, WebCryptoHmacSha256, WebCryptoSha2Hash, aes256CbcDecrypt, aes256CbcEncrypt, base58check, createCipher, createHashRipemd160, createHmacSha256, createSha2Hash, createUnsecuredToken, decodeToken, decryptECIES, encryptECIES, equalConstTime, getCryptoLib, getRandomBytes, hashRipemd160, hashSha256, hashSha512, hmacSha256, isNodeCryptoAvailable, isSubtleCryptoAvailable, privateKeyToAddress, publicKeyToAddress, sharedSecretToKeys };
//# sourceMappingURL=stacks-crypto.esm.js.map
