/// <reference types="node" />
export interface TokenInterface {
    header: {
        [key: string]: string | Json;
        alg?: any;
        typ?: any;
    };
    payload: {
        [key: string]: Json;
        iss?: any | string;
        jti?: any | string;
        iat?: any | string | number;
        exp?: any | string | number;
    } | string;
    signature: string;
}
export declare type Json = string | number | boolean | null | {
    [property: string]: Json;
} | Json[];
export declare function decodeToken(token: string | TokenInterface): TokenInterface | undefined;
export declare function createUnsecuredToken(payload: Json): string;
export interface SignedToken {
    header: string[];
    payload: string;
    signature: string[];
}
export declare class TokenSigner {
    tokenType: string;
    rawPrivateKey: string;
    constructor(signingAlgorithm: string, rawPrivateKey: string);
    header(header?: {}): {
        typ: string;
        alg: string;
    };
    sign(payload: Json): Promise<string>;
    sign(payload: Json, expanded: true, customHeader?: Json): Promise<SignedToken>;
    sign(payload: Json, expanded: false, customHeader?: Json): Promise<string>;
    createWithSignedHash(payload: Json, expanded: boolean, header: {
        typ: string;
        alg: string;
    }, signingInput: string, signingInputHash: Buffer): Promise<SignedToken | string>;
}
